#include <shaders.h>
#include <stdarg.h>
#define SL_IMPLEMENTATION
#include <slibs/slibs.h>
#include <opengl.h>

uint32_t compile_shader(const char* path, int32_t type) {
    sl_string source = { 0 };
    sl_read_file(path, &source);
    const char* source_cstr = sl_c_str(source);

    uint32_t shader = glCreateShader(type);
    glShaderSource(shader, 1, &source_cstr, NULL);
    glCompileShader(shader);

    sl_str_free(source);

    int success;
    char info_log[512];
    glGetShaderiv(shader, GL_COMPILE_STATUS, &success);
    if(!success) {
        glGetShaderInfoLog(shader, 512, NULL, info_log);
        printf("[Shader %d, %s] Failed to compile: %s\n", shader, path, info_log);
        exit(-1);
    } else {
        printf("[Shader %d, %s] Compiled successfully\n", shader, path);
    }

    return shader;
}

uint32_t load_shader_program(uint32_t shader, ...) {
    sl_vec(uint32_t) shaders = { 0 };

    va_list args;
    va_start(args, shader);
    while(shader != 0) {
        sl_vec_push(shaders, shader);
        shader = va_arg(args, uint32_t);
    }
    va_end(args);

    uint32_t program = glCreateProgram();
    for(sl_vec_it(shader, shaders)) {
        glAttachShader(program, *shader);
        printf("[Shader %d] Attached to program %d\n", *shader, program);
    }
    glLinkProgram(program);

    int success;
    char info_log[512];
    glGetProgramiv(program, GL_LINK_STATUS, &success);
    if(!success) {
        glGetProgramInfoLog(program, 512, NULL, info_log);
        printf("[Program %d] Failed to link: %s\n", program, info_log);
        exit(-1);
    } else {
        printf("[Program %d] Linked successfully\n", program);
    }

    for(sl_vec_it(shader, shaders)) {
        glDeleteShader(*shader);
        printf("[Shader %d] Deleted successfully\n", *shader);
    } 

    return program;
}
